using ArgParse

###############################################################################
#
#  Parsing command line
#
###############################################################################

function parse_commandline()
    settings = ArgParseSettings()

    @add_arg_table settings begin
        "--tol"
            help = "set numerical tolerance for the SDP solver"
            arg_type = Float64
            default = 1e-6
        "--iterations"
            help = "set maximal number of iterations for the SDP solver"
            arg_type = Int
            default = 50000
        "--upper-bound"
            help = "Set an upper bound for the spectral gap"
            arg_type = Float64
            default = Inf
        "--cpus"
            help = "Set number of cpus used by solver"
            arg_type = Int
            required = false
        "-N"
            help = "Consider elementary matrices EL(N)"
            arg_type = Int
            default = 2
        "-p"
            help = "Matrices over field of p-elements (p=0 => over ZZ)"
            arg_type = Int
            default = 0
        "--radius"
            help = "Radius of ball B_r(e,S) to find solution over"
            arg_type = Int
            default = 2
        "-X"
            help = "Consider EL(N, ZZ⟨X⟩)"
            action = :store_true
        "--warmstart"
            help = "Use warmstart.jld as the initial guess for SCS"
            action = :store_true
    end

    return parse_args(settings)
end
const PARSEDARGS = parse_commandline()

include("CPUselect.jl")
set_parallel_mthread(PARSEDARGS, workers=true)

using Nemo
using SCS.SCSSolver
using PropertyT
using Groups

include("groups/speciallinear.jl")

function main(GROUP, parsed_args)

    radius = parsed_args["radius"]
    tol = parsed_args["tol"]
    iterations = parsed_args["iterations"]
    upper_bound = parsed_args["upper-bound"]
    warm = parsed_args["warmstart"]

    name, N = GROUP.groupname(parsed_args)
    G, S = GROUP.generatingset(parsed_args)

    name = "$(name)_r$radius"
    isdir(name) || mkdir(name)

    logger = PropertyT.setup_logging(joinpath(name, "$(upper_bound)"))

    info(logger, "Group:       $name")
    info(logger, "Iterations:  $iterations")
    info(logger, "Precision:   $tol")
    info(logger, "Upper bound: $upper_bound")
    info(logger, G)
    info(logger, "Symmetric generating set of size $(length(S))")
    info(logger, "Threads:     $(Threads.nthreads())")
    info(logger, "Workers:     $(workers())")

    Id = one(G)

    solver = SCSSolver(eps=tol, max_iters=iterations, linearsolver=SCS.Direct, alpha=1.95, acceleration_lookback=1)

    PropertyT.check_property_T(name, S, Id, solver, upper_bound, tol, radius, warm)
    return 0
end

main(SpecialLinear, PARSEDARGS)
